﻿using System;
using System.Collections.Generic;
using Yaskawa.PLCIInterface;

namespace PLCi_Simple_Watch_Window
{
    public class MyPLCi
    {
        public PLCIHandler handler_ = new PLCIHandler();
        public List<string> varList_ = new List<string>();                      // not used in application - for reference only. See ReadVariables() below.
        public string subscriptionName_ = "MyMPiecStruct";                      // not used in application - for reference only. See ReadVariables() below.

        public const string mySubscription_ = "@GlobalVariables.MyStruct";      // "MyStruct" struct on MPiec
        public myStruct myStructData_ = new myStruct();                         //  myStructData_ matches layout of MyStruct struct in MotionWorks IEC (same names and types)
        public List<string> structVarList_ = new List<string>();
        public static bool firstConnection;                                     // indicates whether it is the first time the app is connecting to a given controller
        public static string currentIPAddress;                                  // IP Address of current connection


        // connect myStructData from C# to MachineSimple struct on controller
        public void ConnectVariables(string ipAddress)
        {
            handler_.DisconnectDispose();       // clear out any previous connections
            try
            {
                handler_.Connect(ipAddress);
                structVarList_ = StructureConverter.GenerateVariables(myStructData_, mySubscription_);
                handler_.Subscribe(mySubscription_, structVarList_);
            }
            catch (System.NullReferenceException e)     // null reference will occur after ~30 seconds if MPiecComm is unable to connect to controller
            {
                throw e;  // throw to WatchWindow.cs so a MessageBox message can be displayed
            }
            if (ipAddress != currentIPAddress)
                firstConnection = true;         // first time connecting to a new controller
            currentIPAddress = ipAddress;

            if (firstConnection)    // print out some additional information to the console if it is the first time the app is connecting to the PLC
            {
                Console.WriteLine("IP Address: {0}", ipAddress);
                //  Print names of all variables found in struct
                Console.WriteLine("Variables in user defined structure ...");
                int count = 0;
                foreach (string i in structVarList_)
                {
                    count++;
                    Console.WriteLine("Variable {0}: {1}", count, i);
                }
                firstConnection = false;
            }
        }

        // disconnect from the PLC, remove any subscriptions, and destroy the MPiecComm object
        public void Disconnect()
        {
            handler_.DisconnectDispose();
        }

        //   Reads from structure on PLC and returns a matching structure on the C# side
        public myStruct ReadStruct()
        {
            IList<object> structValues = handler_.ReadVariableValues(mySubscription_);
            myStructData_ = (myStruct)StructureConverter.PopulateStructValues(myStructData_, mySubscription_, structVarList_, structValues);
            return myStructData_;
        }

        public void WriteStruct(myStruct MyData)
        {
            myStructData_.XPos = MyData.XPos;
            myStructData_.YPos = MyData.YPos;
            myStructData_.ZPos = MyData.ZPos;
            myStructData_.Feed = MyData.Feed;
            myStructData_.CycleStart = MyData.CycleStart;
            myStructData_.CycleStop = MyData.CycleStop;
            myStructData_.StartMove = MyData.StartMove;
            IList<object> structValues = StructureConverter.GenerateVariableValues(myStructData_);
            handler_.WriteVariableValues(structVarList_, structValues);
            return;
        }

        //   Writes to a single variable - Not used in this example project, shown for reference only.
        public void WriteVariable()
        {
            handler_.WriteVariableValue("@GlobalVariables.StartMove", false);
            return;
        }

        //   Reads a single variable - Not used in this example project, shown for reference only.
        public IList<object> ReadVariables()
        {
            IList<object> variables = handler_.ReadVariableValues(subscriptionName_);
            return variables;
        }
    }

    //   Variable names in myStruct must match those on the MPiec exactly (case sensitive!)
    //   Also, note that types between MotionWorks IEC and C# are not consistent
    //   LREAL = double
    //   INT = short
    public class myStruct
    {
        public double XPos;
        public double YPos;
        public double ZPos;
        public double Feed;
        public bool CycleStart;
        public bool CycleStop;
        public bool StartMove;
    }
}
